<?php
/**
 * Script file for Cookie Control 8 module
 *
 * @package     CookieControl8.CIVIC
 * @subpackage  Modules
 * @author      Civic Team <queries@civicuk.com>
 * @copyright   Copyright (C) 2018 Open Source Matters, Inc. All rights reserved.
 * @license     GNU/GPL v2 or later http://www.gnu.org/licenses/gpl-2.0.html
 *
 * Websites: https://www.civicuk.com
 * Documentation:  https://www.civicuk.com/cookie-control/v8/documentation
 */

// No direct access
defined('_JEXEC') or die;

/**
 * Script class for Cookie Control 8 module
 * @since       1.12
 */
class mod_CookieControlInstallerScript
{
	/**
	 * The extension name. This should be set in the installer script.
	 *
	 * @var    string
	 * @since  1.12
	 */
	protected $extension = 'mod_cookiecontrol';

	/**
	 * Method to install the extension
	 * @param   InstallerAdapter $parent is the class calling this method
	 *
	 * @return void
	 */
	public function install($parent)
	{
		echo '<p>The module has been installed.</p>';
	}

	/**
	 * Method to uninstall the extension
	 * @param   InstallerAdapter $parent is the class calling this method
	 *
	 * @return void
	 */
	public function uninstall($parent)
	{
		echo '<p>The module has been uninstalled.</p>';
	}

	/**
	 * Method to update the extension
	 * @param   InstallerAdapter $parent is the class calling this method
	 *
	 * @return void
	 */
	public function update($parent)
	{
		echo '<p>The module has been updated to version' . $parent->get('manifest')->version . '.</p>';
	}

	/**
	 * Method to run before an install/update/uninstall method
	 * @param   string           $type   is the type of change (install, update or discover_install)
	 * @param   InstallerAdapter $parent is the class calling this method
	 *
	 * @return void
	 */
	public function preflight($type, $parent)
	{
		if (strtolower($type) === 'update')
		{
			$manifest   = $this->getItemArray('manifest_cache', '#__extensions', 'element', JFactory::getDbo()->quote($this->extension));
			$oldVersion = $manifest['version'];

			if (version_compare($oldVersion, '1.12', '<'))
			{
				$instancesId = $this->getInstances($this->extension);

				if ($instancesId && (count($instancesId) > 0))
				{
					foreach ($instancesId as $id)
					{
						$paramArray = array();
						$optionalCookies = json_decode($this->getParam('optionalCookies', $id), true);

						if (!empty($optionalCookies) && count($optionalCookies) > 0)
						{
							$arraySize = count($optionalCookies['optionalCookiesName']);

							for ($i = 0; $i < $arraySize; $i++)
							{
								$paramArray['optionalCookiesSubform']['optionalCookiesSubform' . $i] = array();

								foreach ($optionalCookies as $key => $val)
								{
									$paramArray['optionalCookiesSubform']['optionalCookiesSubform' . $i][$key] = $val[$i];
								}
							}
						}

						$this->setParams($paramArray, 'edit', $id);
					}
				}
			}
		}
	}

	/**
	 * Method to run after an install/update/uninstall method
	 * @param   string           $type   is the type of change (install, update or discover_install)
	 * @param   InstallerAdapter $parent is the class calling this method
	 *
	 * @return void
	 */
	public function postflight($type, $parent)
	{
	}

	/**
	 * Builds a standard select query to produce better DRY code in this script.
	 * This should produce a single unique cell which is json encoded - it will then
	 * return an associated array with this data in.
	 *
	 * @param   string  $element     The element to get from the query
	 * @param   string  $table       The table to search for the data in
	 * @param   string  $column      The column of the database to search from
	 * @param   mixed   $identifier  The integer id or the already quoted string
	 *
	 * @return  array  Associated array containing data from the cell
	 *
	 */
	protected function getItemArray($element, $table, $column, $identifier)
	{
		// Get the DB and query objects
		$db = \JFactory::getDbo();

		// Build the query
		$query = $db->getQuery(true)
			->select($db->quoteName($element))
			->from($db->quoteName($table))
			->where($db->quoteName($column) . ' = ' . $identifier);
		$db->setQuery($query);

		// Load the single cell and json_decode data.
		return json_decode($db->loadResult(), true);
	}

	/**
	 * Gets each instance of a module in the #__modules table
	 *
	 * @param   string  $identifier  Module identifier
	 *
	 * @return  array  An array of ID's of the extension
	 *
	 * @since   3.6
	 */
	protected function getInstances($identifier)
	{
		$db = \JFactory::getDbo();
		$query = $db->getQuery(true);

		// Select the item(s) and retrieve the id
		$query->select($db->quoteName('id'));

		$query->from($db->quoteName('#__modules'))
			->where($db->quoteName('module') . ' = ' . $db->quote($identifier));

		// Set the query and obtain an array of id's
		return $db->setQuery($query)->loadColumn();
	}

	/**
	 * Gets parameter value in the extensions row of the extension table
	 *
	 * @param   string   $name  The name of the parameter to be retrieved
	 * @param   integer  $id    The id of the item in the Param Table
	 *
	 * @return  string  The parameter desired
	 *
	 */
	protected function getParam($name, $id = 0)
	{
		if (!is_numeric($id) || $id == 0)
		{
			// Return false if there is no item given
			return false;
		}

		$params = $this->getItemArray('params', '#__modules', 'id', $id);

		return $params[$name];
	}

	/**
	 * Sets parameter values in the extensions row of the extension table. Note that the
	 * this must be called separately for deleting and editing. Note if edit is called as a
	 * type then if the param doesn't exist it will be created
	 *
	 * @param   array    $paramArray  The array of parameters to be added/edited/removed
	 * @param   string   $type        The type of change to be made to the param (edit/remove)
	 * @param   integer  $id          The id of the item in the relevant table
	 *
	 * @return  boolean  True on success
	 *
	 */
	protected function setParams($paramArray = null, $type = 'edit', $id = 0)
	{
		if (!is_numeric($id) || $id == 0)
		{
			// Return false if there is no valid item given.
			return false;
		}

		$params = $this->getItemArray('params', '#__modules', 'id', $id);

		if ($paramArray)
		{
			foreach ($paramArray as $name => $value)
			{
				if ($type === 'edit')
				{
					// Add or edit the new variable(s) to the existing params
					if (is_array($value))
					{
						// Convert an array into a json encoded string
						$params[(string) $name] = array_values($value);
					}
					else
					{
						$params[(string) $name] = (string) $value;
					}
				}
				elseif ($type === 'remove')
				{
					// Unset the parameter from the array
					unset($params[(string) $name]);
				}
			}
		}

		// Store the combined new and existing values back as a JSON string
		$paramsString = json_encode($params);

		$db = \JFactory::getDbo();
		$query = $db->getQuery(true)
			->update($db->quoteName('#__modules'))
			->set('params = ' . $db->quote($paramsString))
			->where('id = ' . $id);

		// Update table
		$db->setQuery($query)->execute();

		return true;
	}
}
